import click
import subprocess
import tempfile
import os
from convisoappsec.flowcli.context import pass_flow_context
from datetime import datetime
from convisoappsec.flowcli.requirements_verifier import RequirementsVerifier
from convisoappsec.flowcli import help_option
from convisoappsec.flowcli.common import (asset_id_option, project_code_option)


@click.command()
@project_code_option(required=False)
@asset_id_option(required=False)
@project_code_option(
    help="Not required when --no-send-to-flow option is set",
    required=False
)
@click.option(
    '-r',
    '--repository-dir',
    default=".",
    show_default=True,
    type=click.Path(
        exists=True,
        resolve_path=True,
    ),
    required=False,
    help="The source code repository directory.",
)
@click.option(
    "--send-to-flow/--no-send-to-flow",
    default=True,
    show_default=True,
    required=False,
    help="""Enable or disable the ability of send analysis result
    reports to flow. When --send-to-flow option is set the --project-code
    option is required""",
    hidden=True
)
@click.option(
    "--custom-sca-tags",
    hidden=True,
    required=False,
    multiple=True,
    type=(str, str),
    help="""It should be passed as <repository_name> <image_tag>. It accepts multiple values"""
)
@click.option(
    "--scanner-timeout",
    hidden=True,
    required=False,
    default=7200,
    type=int,
    help="Set timeout for each scanner"
)
@click.option(
    "--parallel-workers",
    hidden=True,
    required=False,
    default=2,
    type=int,
    help="Set max parallel workers"
)
@click.option(
    "--deploy-id",
    default=None,
    required=False,
    hidden=True,
    envvar=("CONVISO_DEPLOY_ID", "FLOW_DEPLOY_ID")
)
@click.option(
    '--experimental',
    default=False,
    is_flag=True,
    hidden=True,
    help="Enable experimental features.",
)
@click.option(
    "--company-id",
    required=False,
    envvar=("CONVISO_COMPANY_ID", "FLOW_COMPANY_ID"),
    help="Company ID on Conviso Platform",
)
@click.option(
    '--asset-name',
    required=False,
    envvar=("CONVISO_ASSET_NAME", "FLOW_ASSET_NAME"),
    help="Provides a asset name.",
)
@click.option(
    '--vulnerability-auto-close',
    default=False,
    is_flag=True,
    hidden=True,
    help="Enable auto fixing vulnerabilities on cp.",
)
@click.option(
    '--from-ast',
    default=False,
    is_flag=True,
    hidden=True,
    help="Internal use only.",
)
@help_option
@pass_flow_context
@click.pass_context
def generate(context, flow_context, project_code, asset_id, company_id, repository_dir, send_to_flow, custom_sca_tags,
             scanner_timeout, parallel_workers, deploy_id, experimental, asset_name, vulnerability_auto_close, from_ast):

    context.params['company_id'] = context.params.get('company_id') or company_id

    if not context.params['company_id']:
        log_func("Invalid company_id: {company_id}. Exiting.".format(company_id=company_id))
        return

    if not from_ast:
        prepared_context = RequirementsVerifier.prepare_context(context)

        params_to_copy = [
            'project_code', 'asset_id', 'repository_dir', 'send_to_flow',
            'deploy_id', 'custom_sca_tags', 'scanner_timeout', 'parallel_workers',
            'experimental', 'asset_name', 'vulnerability_auto_close'
        ]

        for param_name in params_to_copy:
            context.params[param_name] = (
                    locals()[param_name] or prepared_context.params[param_name]
            )

    log_func("Generating SBOM file...")
    asset_name = context.params['asset_name']
    temp_dir = tempfile.gettempdir()
    timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
    sanitized_asset_name = asset_name.replace(" ", "_").replace("(", "").replace(")", "")
    file_name = os.path.join(temp_dir, "sbom_{asset_name}_{timestamp}.json".format(
        asset_name=sanitized_asset_name, timestamp=timestamp
    ))

    command = [
        f"syft scan {repository_dir} -o cyclonedx-json={file_name}"
    ]

    try:
        subprocess.run(command, shell=True, check=True, capture_output=True)
        log_func("SBOM file generated successfully!")
    except subprocess.CalledProcessError as error:
        log_func(f"Error generating SBOM file: {error}")
        return

    asset_id = asset_id or context.params.get('asset_id')

    send_sbom_file_to_csc(company_id=company_id, asset_id=asset_id, file=file_name)


@pass_flow_context
def send_sbom_file_to_csc(flow_context, company_id, asset_id, file):
    conviso_api = flow_context.create_conviso_graphql_client()
    api_key = flow_context.key

    log_func(f"Sending sbom to conviso platform ...")

    return conviso_api.sbom.send_sbom_file(company_id=company_id, asset_id=asset_id, file_path=file, api_key=api_key)


def log_func(msg, new_line=True):
    click.echo(click.style(msg), nl=new_line, err=True)